/*
pid.cc

This C++ file defines member functions for the class PID_MODULE
It was generated with rcsdesign
with template version 1.1

MODIFICATIONS:
12-Sep-99	Created with RCS-Design tool.

*/

// Include Files
#include "rcs.hh" 	// Common RCS definitions
#include "nml_mod.hh" 	// NML_MODULE definitions
#include "pid.hh" 	// PID_MODULE definitions
// RCS-Design-MERGE-DISABLE Edits to the following area will NOT be preserved by the RCS-Design tool.


#include "pidn.hh" 	// NML Commands and Status definitions for pid
// RCS-Design-MERGE-ENABLE Edits after this line will be preserved by the RCS-Design tool.


// Constructor
PID_MODULE::PID_MODULE(int _is_base_class)
{

	// Initialize the NML channels if this module is not being used as the base class for another module.
	if(!_is_base_class)
	{
		INITIALIZE_NML();
	}
	// Add additional code to initialize the module here.
	pid_ctrl = new PID_CTRL();
	if (pid_ctrl->pid_ctrl_err != PID_CTRL_NO_ERROR)
	  {
	    logError("Error in creating PID_MODULE object!!!");
	    status = RCS_ERROR; // let my supervisor know I have a problem.
	  }

}

// Overloaded Virtual Functions

/*
INITIALIZE_NML

The INITIALIZE_NML function is ussually called only once from within the
constructor. It should not be called if a derived class will also call it.

*/
void PID_MODULE::INITIALIZE_NML()
{
	// RCS-Design-MERGE-DISABLE Edits to the following area will NOT be preserved by the RCS-Design tool.

	setErrorLogChannel(new NML(nmlErrorFormat, "errlog", "pid", "pid_module.nml"));
	setCmdChannel(new RCS_CMD_CHANNEL(pidFormat, "pid_cmd", "pid", "pid_module.nml"));
	pid_status = new PID_STATUS();
	setStatChannel(new RCS_STAT_CHANNEL(pidFormat, "pid_sts", "pid", "pid_module.nml"), pid_status);
	// RCS-Design-MERGE-ENABLE Edits after this line will be preserved by the RCS-Design tool.


}

/*
PRE_PROCESS

The PRE_PROCESS function is called every cycle after the command and
subordinates status have been read but before DECISION_PROCESS is called.
It is intended to be used for tasks such as sensory processing that should
be performed every cycle regardless of the current command or state.
*/
void PID_MODULE::PRE_PROCESS()
{
	// Pre-Processing Code
  // Read the plant output 
  plant_output = 0.0;
}

/*
DECISION_PROCESS

The DECISION_PROCESS function is called every cycle as long as there is a non-zero command.
It is expected to call a command function based on commandInData->type.
*/
void PID_MODULE::DECISION_PROCESS()
{
	switch(commandInData->type)
	{
		// RCS-Design-MERGE-DISABLE Edits to the following area will NOT be preserved by the RCS-Design tool.

	case PID_INIT_TYPE:
		INIT((PID_INIT *)commandInData);
		break;

	case PID_HALT_TYPE:
		HALT((PID_HALT *)commandInData);
		break;

	case PID_RUN_TYPE:
		RUN((PID_RUN *)commandInData);
		break;

	case PID_SET_GAINS_TYPE:
		SET_GAINS((PID_SET_GAINS *)commandInData);
		break;
		// RCS-Design-MERGE-ENABLE Edits after this line will be preserved by the RCS-Design tool.


	default:
		logError("The command %d is not recognized.",commandInData->type);
		break;
	}
}

/*
POST_PROCESS

The POST_PROCESS function is called every cycle after DECISION_PROCESS is called
but before the status and the subordinates commands  have been written.
It is intended to be used for tasks such as output filters that should
be performed every cycle regardless of the current command or state.
*/
void PID_MODULE::POST_PROCESS()
{
	// Post-Processing Code
  pid_status->plant_output = plant_output;
  pid_status->ctrl_output = ctrl_output;
}

// Command Functions

/*
INIT

Parameter(s):
PID_INIT *cmd_in -- NML Message sent from superior.

Most Modules will have an INIT command.
The INIT function is expected to initialize any variables that may be
in an uninitialized or unknown state, send INIT commands to its subordinates,
wait for the subordinates to be DONE and then inform its superior that it is done.
The state tables should use the STATE_MATCH macro so the diagnostics tool can 
highlight the current line in the state table.
*/
void PID_MODULE::INIT(PID_INIT *cmd_in)
{
	// RCS-Design-MERGE-DISABLE Edits to the following area will NOT be preserved by the RCS-Design tool.

	// RCS-Design-MERGE-ENABLE Edits after this line will be preserved by the RCS-Design tool.


	if(STATE_MATCH(NEW_COMMAND))
	{
		// Send an INIT command to all subordinates.
		// RCS-Design-MERGE-DISABLE Edits to the following area will NOT be preserved by the RCS-Design tool.

		// RCS-Design-MERGE-ENABLE Edits after this line will be preserved by the RCS-Design tool.


		stateNext(S1);
		// Reinitialize variables here.

	}
	// Wait for all subordinates to report done.
	// RCS-Design-MERGE-DISABLE Edits to the following area will NOT be preserved by the RCS-Design tool.

	else if(STATE_MATCH(S1,
		1))
	// RCS-Design-MERGE-ENABLE Edits after this line will be preserved by the RCS-Design tool.

	{
	  pid_ctrl->init_pid(cmd_in->sample_time); 
	  status = RCS_DONE;
	  stateNext(S2);
	}
	else if(STATE_MATCH(S2))
	{
		// Idle State
	}
}

/*
HALT

Parameter(s):
PID_HALT *cmd_in -- NML Message sent from superior.

Most Modules will have an HALT command.
The HALT function is expected to stop any motion or command execution,
send HALT commands to its subordinates,
wait for the subordinates to be DONE and then inform its superior that it is done.
The state tables should use the STATE_MATCH macro so the diagnostics tool can 
highlight the current line in the state table.
*/
void PID_MODULE::HALT(PID_HALT *cmd_in)
{
	// RCS-Design-MERGE-DISABLE Edits to the following area will NOT be preserved by the RCS-Design tool.

	// RCS-Design-MERGE-ENABLE Edits after this line will be preserved by the RCS-Design tool.


	if(STATE_MATCH(NEW_COMMAND))
	{
		//Send a HALT command to all subordinates.
		// RCS-Design-MERGE-DISABLE Edits to the following area will NOT be preserved by the RCS-Design tool.

		// RCS-Design-MERGE-ENABLE Edits after this line will be preserved by the RCS-Design tool.

		stateNext(S1);
	}
	// Wait for all subordinates to report done.
	// RCS-Design-MERGE-DISABLE Edits to the following area will NOT be preserved by the RCS-Design tool.

	else if(STATE_MATCH(S1,
		1))
	// RCS-Design-MERGE-ENABLE Edits after this line will be preserved by the RCS-Design tool.

	{
		status = RCS_DONE;
		stateNext(S2);
	}
	else if(STATE_MATCH(S2))
	{
		// Idle State
	}
}

/*
RUN

Parameter(s):
PID_RUN *cmd_in -- NML Message sent from superior.

*/
void PID_MODULE::RUN(PID_RUN *cmd_in)
{
	// Put state table for PID_RUN here.
  if(STATE_MATCH(NEW_COMMAND))
    {
      status = RCS_EXEC;
      stateNext(S1);
    }
  else if(STATE_MATCH(S1))
    {
      ctrl_output = pid_ctrl->control(plant_output - cmd_in->reference_input);
    }
  else if(STATE_MATCH(S2))
    {
      // Idle State
    }
}

/*
SET_GAINS

Parameter(s):
PID_SET_GAINS *cmd_in -- NML Message sent from superior.

*/
void PID_MODULE::SET_GAINS(PID_SET_GAINS *cmd_in)
{
	// Put state table for PID_SET_GAINS here.
  if(STATE_MATCH(NEW_COMMAND))
    {
      status = RCS_EXEC;
      stateNext(S1);
    }
  else if(STATE_MATCH(S1))
    {
      pid_ctrl->set_gains(cmd_in->Kp, cmd_in->Ki, cmd_in->Kd); 
      status = RCS_DONE;
      stateNext(S2);
 
    }
  else if(STATE_MATCH(S2))
    {
      // Idle State
    }
}

